//Nice way to import !
use hello::{
    ThreadPool,
};
use std::{
    fs,
    io::{prelude::*, BufReader},
    net::{TcpListener, TcpStream},
    collections::HashMap,
};
use config::Config;

fn main() {
    //building config
    let cfg = Config::builder()
        .add_source(config::File::with_name("websrv_conf"))
        .build()
        .unwrap();

    let cfg = cfg.try_deserialize::<HashMap<String, String>>().expect("Could not deserialize configuration");
    println!("Starting with configuration: ");
    for conf in &cfg {
        println!("{:?}", conf);
    }

    let nbr_of_threads: usize = cfg
        .get("threads")
        .expect("Threads not found in config")
        .parse()
        .unwrap();

    let default_bind_addr = String::from("0.0.0.0:7878");
    let bind_addr = cfg
        .get("bind_addr")
        .unwrap_or(&default_bind_addr)
        .to_string();

    let default_webpage = String::from("index.html");
    let webpage = cfg
        .get("webpage")
        .unwrap_or(&default_webpage)
        .to_string();

    let listener = TcpListener::bind(bind_addr).unwrap();

    let pool = ThreadPool::new(nbr_of_threads);

    for stream in listener.incoming() {
        let stream = stream.unwrap();
        let webp = webpage.clone();

        pool.execute(move || {
           handle_connection(stream, webp);
        });
    }

    println!("Shutting down.");
}

fn analyze_request_line(line: Vec<&str>) -> HashMap<String, String>{
    let req = line[0];
    let page = line[1];
    let proto = line[2];

    let mut analyzed = HashMap::new();
    analyzed.insert(String::from("req"), req.to_string());
    analyzed.insert(String::from("page"), page.to_string());
    analyzed.insert(String::from("proto"), proto.to_string());

    return analyzed;
}

fn handle_connection(mut stream: TcpStream, webpage: String) {
    let buf_reader = BufReader::new(&mut stream);
    let request_line = buf_reader.lines().next().unwrap().unwrap();
    
    let request_analyze: Vec<&str> = request_line.split_whitespace().collect();
    println!("!{:?}", request_analyze);
    let analyzed = analyze_request_line(request_analyze);

    //We analyzed the request, now to use it

    let (status_line, filename) = match &request_line[..] {
        "GET / HTTP/1.1" => ("HTTP/1.1 200 OK", &webpage[..]),
        _ => ("HTTP/1.1 404 NOT FOUND", "404.html"),
    };


    let contents = fs::read_to_string(format!("./html/{filename}")).unwrap();
    let length = contents.len();

    let response = format!(
        "{status_line}\r\nContent-Length: {length}\r\n\r\n{contents}"
        );
    stream.write_all(response.as_bytes()).unwrap();
}


